import urllib3
from lxml.html import fromstring
from lxml.cssselect import CSSSelector
from typing import TextIO, overload
from urllib.parse import urlparse
from json import dump


class Gooder:
    __captcha: str | None = None
    __headers: dict | None = None

    raw_results: list = []

    def __init__(self) -> None:
        # Init main class
        self.pool = urllib3.PoolManager(num_pools=10)

    def get_captcha_url(self) -> str | None:
        return self.__captcha

    def get_headers(self) -> dict | None:
        return self.__headers

    def parse(
        self,
        query: str,
        page: int = 0,
        ignore_google: bool = True,
        clear_old: bool = True,
    ) -> bool:
        # Add page field on end
        if page:
            query += f"&start={10 * page}"

        # Make request
        r = self.pool.request(
            "GET",
            f"https://google.com/search?q={query}",
            headers={
                "user-agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/100.0.4896.75 Safari/537.36",
            },
        )

        # Save metadata
        self.__headers = r.getheaders()

        # Check on rate limit
        if r.geturl() != f"https://google.com/search?q={query}":
            self.__captcha = r.geturl()
            return False

        self.__captcha = None

        p = fromstring(str(r.data))

        if clear_old:
            self.raw_results = []
        for element in CSSSelector("a")(p):
            url = element.get("href")
            if url is None:
                continue
            if len(url) < 3 or url.startswith("/search?"):
                continue
            if ignore_google and "google" in url:
                continue
            self.raw_results.append([url, element.text_content()])

        return True

    @overload
    def get_hostname(self, links: list[str]) -> list[str | None]:
        ...

    @overload
    def get_hostname(self, links: str) -> str | None:
        ...

    def get_hostname(self, links: str | list[str]) -> str | None | list[str | None]:
        """
        Get hostname(s) from string or list
        """
        if isinstance(links, str):
            return urlparse(links).hostname

        hosts = []
        for url in links:
            hosts.append(urlparse(url).hostname)
        return hosts

    def get_links(self, repeats: bool = False) -> list:
        """
        Get only links from parsing results

        repeats = True, if you want get links with repeats hostnames
        Example: https://google.com/firstpage, https://google.com/secondpage
        """
        urls: list[str] = []
        for element in self.raw_results:
            url = element[0]
            if not repeats:
                if self.get_hostname(url) in self.get_hostname(urls):
                    continue
            urls.append(url)

        return urls

    def get_titles(self) -> list:
        """
        Get only titles from parsing results
        """
        titles = []
        for element in self.raw_results:
            end = element[1].find("http")
            if end == -1:
                continue

            titles.append(element[1][:end])

        return titles

    def save_to_file(
        self,
        only_urls: bool = True,
        override: bool = True,
        to_json: bool = False,
        file: str = "urls.txt",
    ) -> TextIO:
        """
        Save parsering results to file
        """
        if override:
            f = open(file, "w+", encoding="utf-8")
        else:
            f = open(file, "a+", encoding="utf-8")

        if not to_json:
            f.writelines(
                "\n".join(self.get_links())
                if only_urls
                else [f"{k} {v}\n" for k, v in dict(self.raw_results).items()]
            )
        else:
            dump(self.get_links() if only_urls else dict(self.raw_results), f)

        return f



# -*- coding: utf-8 -*-
from setuptools import setup

modules = \
['gooder']
install_requires = \
['cssselect>=1.1.0,<2.0.0', 'lxml>=4.8.0,<5.0.0', 'urllib3>=1.26.9,<2.0.0']

setup_kwargs = {
    'name': 'gooder',
    'version': '0.3.0',
    'description': 'Simple Google parser',
    'long_description': '# <b>GOO</b>gle spi<b>DER</b>\nGoogle search engine parser on python3\n\n## Instruction\n> Requirement python 3.10+\n\n> pip install gooder\n\n```python\nfrom gooder import Gooder\n\ngooder = Gooder()\n# Make request on google.com/search?q=Hello+World\nparsed = gooder.parse(query="Hello World")\n\n# Print only result links\nprint(gooder.get_links())\n\n# Print only result titles\nprint(gooder.get_titles())\n\n# Print all results list[tuple[link,title]]\nprint(gooder.raw_results)\n\n# Check on captcha (rate limit)\nif (parsed)\n    # Save urls to json file\n    gooder.save_to_file(only_urls=True, to_json=True, override=True, file="results.json")\n```\n\n## Methods & Fields\n| Method/Field | Args | Example | Result |\n|---|---|---|---|\n| Gooder.parse | query: str, page: int=0, ignore_google: bool=True, clear_old: bool=True | gooder.parse("hello",  clear_old=False) | True \\| False |\n| Gooder.raw_results | **Field** | **Field** | [[link, title], ...] |\n| Gooder.get_links | repeats: bool = False | gooder.get_links() | [unique_link, ...] |\n| Gooder.get_titles | *None* | gooder.get_titles() | [title, title, ...] |\n| Gooder.save_to_file | only_urls: bool = True override: bool = True to_json: bool = False file: str = "urls.txt"  | gooder.save_to_file() | New file with urls |\n| Gooder.get_hostname | links: str \\| list | gooder.get_hostname(https://google.com/) | google.com |\n| Gooder.get_captcha_url | *None* | gooder.get_captcha_url() | *None* \\| google.com/sorry/... |\n| Gooder.get_headers | *None* | gooder.get_headers() | *None* \\| HTTPHeaderDict({...}) |\n\n## Todo:\n + Add proxy manager',
    'author': '9Slavatar',
    'author_email': 'nslavatar@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/9Slavatar/gooder',
    'py_modules': modules,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
